@@ -752,6 +752,15 @@ static GtkWidget *remmina_file_editor_create_check(RemminaFileEditor *gfe, GtkWi
 	return widget;
 }
 
+/**
+ * Create checkbox + gtk_file_chooser for open files and select folders
+ *
+ * The code is wrong, because if the checkbox is not active, the value should be set to NULL
+ * and remove it from the remmina file. The problem is that this function knows nothing about
+ * the remmina file.
+ * This should be rewritten in a more generic way
+ * Please use REMMINA_PROTOCOL_SETTING_TYPE_TEXT
+ */
 static GtkWidget *
 remmina_file_editor_create_chooser(RemminaFileEditor *gfe, GtkWidget *grid, gint row, gint col, const gchar *label,
 				   const gchar *value, gint type, gchar* setting_name)
@@ -1103,6 +1112,7 @@ static void remmina_file_editor_create_behavior_tab(RemminaFileEditor *gfe)
 						remmina_file_get_int(priv->remmina_file, "profile-lock", FALSE), "profile-lock");
 }
 
+#ifdef HAVE_LIBSSH
 static gpointer ssh_tunnel_auth_list[] =
 {
 	"0", N_("Password"),
@@ -1112,6 +1122,7 @@ static gpointer ssh_tunnel_auth_list[] =
 	"4", N_("Kerberos (GSSAPI)"),
 	NULL
 };
+#endif
 
 static void remmina_file_editor_create_ssh_tunnel_tab(RemminaFileEditor *gfe, RemminaProtocolSSHSetting ssh_setting)
 {
