@@ -286,7 +286,7 @@ static void remmina_main_show_snap_welcome()
 		g_print("  but we can’t find the secret plugin inside the SNAP.\n");
 		need_snap_interface_connections = TRUE;
 	} else {
-		if (!remmina_secret_plugin->is_service_available()) {
+		if (!remmina_secret_plugin->is_service_available(remmina_secret_plugin)) {
 			g_print("  but we can’t access a secret service. Secret service or SNAP interface connection is missing.\n");
 			need_snap_interface_connections = TRUE;
 		}
@@ -731,6 +731,14 @@ void remmina_main_on_action_connection_connect(GSimpleAction *action, GVariant *
 	if (remminafile == NULL)
 		return;
 
+	if (remmina_pref_get_boolean("use_primary_password")
+			&& remmina_pref_get_boolean("lock_connect")
+			&& remmina_unlock_new(remminamain->window) == 0)
+		return;
+	if (remmina_file_get_int (remminafile, "profile-lock", FALSE) == 1
+			&& remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	remmina_file_touch(remminafile);
 	rcw_open_from_filename(remminamain->priv->selected_filename);
 
@@ -743,9 +751,6 @@ void remmina_main_on_action_connection_external_tools(GSimpleAction *action, GVa
 	if (!remminamain->priv->selected_filename)
 		return;
 
-	if (remmina_unlock_new(remminamain->window) == 0)
-		return;
-
 	remmina_external_tools_from_filename(remminamain, remminamain->priv->selected_filename);
 }
 
@@ -770,6 +775,12 @@ void remmina_main_on_action_application_mpchange(GSimpleAction *action, GVariant
 	username = domain = group = "";
 
 	remminafile = NULL;
+
+	if (remmina_pref_get_boolean("use_primary_password")
+			&& remmina_pref_get_boolean("lock_edit")
+			&& remmina_unlock_new(remminamain->window) == 0)
+		return;
+
 	if (remminamain->priv->selected_filename) {
 		remminafile = remmina_file_load(remminamain->priv->selected_filename);
 		if (remminafile != NULL) {
@@ -779,9 +790,6 @@ void remmina_main_on_action_application_mpchange(GSimpleAction *action, GVariant
 		}
 	}
 
-	if (remmina_unlock_new(remminamain->window) == 0)
-		return;
-
 	remmina_mpchange_schedule(TRUE, group, domain, username, "");
 
 	if (remminafile != NULL)
@@ -791,9 +799,13 @@ void remmina_main_on_action_application_mpchange(GSimpleAction *action, GVariant
 void remmina_main_on_action_connection_new(GSimpleAction *action, GVariant *param, gpointer data)
 {
 	TRACE_CALL(__func__);
+	if (kioskmode && kioskmode == TRUE)
+		return;
 	GtkWidget *widget;
 
-	if (remmina_unlock_new(remminamain->window) == 0)
+	if (remmina_pref_get_boolean("use_primary_password")
+			&& remmina_pref_get_boolean("lock_edit")
+			&& remmina_unlock_new(remminamain->window) == 0)
 		return;
 
 	widget = remmina_file_editor_new();
@@ -848,12 +860,26 @@ void remmina_main_on_action_connection_copy(GSimpleAction *action, GVariant *par
 	TRACE_CALL(__func__);
 	GtkWidget *widget;
 
-	if (remmina_unlock_new(remminamain->window) == 0)
+	if (remmina_pref_get_boolean("use_primary_password")
+			&& remmina_unlock_new(remminamain->window) == 0)
 		return;
 
 	if (!remminamain->priv->selected_filename)
 		return;
 
+	RemminaFile *remminafile = remmina_file_load(remminamain->priv->selected_filename);
+
+	if (((remmina_pref_get_boolean("lock_edit")
+				&& remmina_pref_get_boolean("use_primary_password"))
+				|| remmina_file_get_int (remminafile, "profile-lock", FALSE))
+			&& remmina_unlock_new(remminamain->window) == 0)
+		return;
+
+	if (remminafile) {
+		remmina_file_free(remminafile);
+		remminafile = NULL;
+	}
+
 	widget = remmina_file_editor_new_copy(remminamain->priv->selected_filename);
 	if (widget) {
 		g_signal_connect(G_OBJECT(widget), "destroy", G_CALLBACK(remmina_main_file_editor_destroy), remminamain);
@@ -870,12 +896,22 @@ void remmina_main_on_action_connection_edit(GSimpleAction *action, GVariant *par
 	TRACE_CALL(__func__);
 	GtkWidget *widget;
 
-	if (remmina_unlock_new(remminamain->window) == 0)
-		return;
-
 	if (!remminamain->priv->selected_filename)
 		return;
 
+	RemminaFile *remminafile = remmina_file_load(remminamain->priv->selected_filename);
+
+	if (remmina_pref_get_boolean("use_primary_password")
+			&& (remmina_pref_get_boolean("lock_edit")
+				|| remmina_file_get_int (remminafile, "profile-lock", FALSE))
+			&& remmina_unlock_new(remminamain->window) == 0)
+		return;
+
+	if (remminafile) {
+		remmina_file_free(remminafile);
+		remminafile = NULL;
+	}
+
 	widget = remmina_file_editor_new_from_filename(remminamain->priv->selected_filename);
 	if (widget) {
 		gtk_window_set_transient_for(GTK_WINDOW(widget), remminamain->window);
@@ -890,20 +926,29 @@ void remmina_main_on_action_connection_delete(GSimpleAction *action, GVariant *p
 {
 	TRACE_CALL(__func__);
 	GtkWidget *dialog;
-	gchar *delfilename;
 
 	if (!remminamain->priv->selected_filename)
 		return;
 
-	if (remmina_unlock_new(remminamain->window) == 0)
+	RemminaFile *remminafile = remmina_file_load(remminamain->priv->selected_filename);
+
+	if (((remmina_pref_get_boolean("lock_edit")
+				&& remmina_pref_get_boolean("use_primary_password"))
+				|| remmina_file_get_int (remminafile, "profile-lock", FALSE))
+			&& remmina_unlock_new(remminamain->window) == 0)
 		return;
 
+	if (remminafile) {
+		remmina_file_free(remminafile);
+		remminafile = NULL;
+	}
+
 	dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
 					_("Are you sure you want to delete “%s”?"), remminamain->priv->selected_name);
 	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_YES) {
-		delfilename = g_strdup(remminamain->priv->selected_filename);
+		gchar *delfilename = g_strdup(remminamain->priv->selected_filename);
 		remmina_file_delete(delfilename);
-		g_free(delfilename);
+		g_free(delfilename), delfilename = NULL;
 		remmina_icon_populate_menu();
 		remmina_main_load_files();
 	}
@@ -935,8 +980,10 @@ void remmina_main_on_action_application_preferences(GSimpleAction *action, GVari
 		tab_num = 0;
 	}
 
-	if (remmina_unlock_new(remminamain->window) == 0)
+	if (remmina_pref_get_boolean("use_primary_password")
+			&& remmina_unlock_new(remminamain->window) == 0)
 		return;
+
 	GtkWidget *widget = remmina_pref_dialog_new(tab_num, remminamain->window);
 
 	gtk_widget_show_all(widget);
@@ -1005,7 +1052,7 @@ static void remmina_main_import_file_list(GSList *files)
 	for (element = files; element; element = element->next) {
 		path = (gchar *)element->data;
 		plugin = remmina_plugin_manager_get_import_file_handler(path);
-		if (plugin && (remminafile = plugin->import_func(path)) != NULL && remmina_file_get_string(remminafile, "name")) {
+		if (plugin && (remminafile = plugin->import_func(plugin, path)) != NULL && remmina_file_get_string(remminafile, "name")) {
 			remmina_file_generate_filename(remminafile);
 			remmina_file_save(remminafile);
 			imported = TRUE;
@@ -1074,7 +1121,7 @@ void remmina_main_on_action_tools_export(GSimpleAction *action, GVariant *param,
 		dialog = gtk_file_chooser_dialog_new(plugin->export_hints, remminamain->window,
 						     GTK_FILE_CHOOSER_ACTION_SAVE, _("_Save"), GTK_RESPONSE_ACCEPT, NULL);
 		if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
-			plugin->export_func(remminafile, gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)));
+			plugin->export_func(plugin, remminafile, gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog)));
 		gtk_widget_destroy(dialog);
 	} else {
 		dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
@@ -1370,7 +1417,7 @@ static gboolean remmina_main_add_tool_plugin(gchar *name, RemminaPlugin *plugin,
 
 	gtk_widget_show(menuitem);
 	gtk_menu_shell_append(GTK_MENU_SHELL(remminamain->menu_popup_full), menuitem);
-	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(tool_plugin->exec_func), NULL);
+	g_signal_connect(G_OBJECT(menuitem), "activate", G_CALLBACK(tool_plugin->exec_func), tool_plugin);
 	return FALSE;
 }
 
@@ -1562,14 +1609,23 @@ void remmina_main_update_file_datetime(RemminaFile *file)
 	remmina_main_load_files();
 }
 
-void remmina_main_show_warning_dialog(const gchar *message)
-{
+void remmina_main_show_dialog(GtkMessageType msg, GtkButtonsType buttons, const gchar* message) {
 	GtkWidget *dialog;
 
 	if (remminamain->window) {
-		dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
-						message, g_get_application_name());
+		dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, msg, buttons, "%s", message);
 		gtk_dialog_run(GTK_DIALOG(dialog));
 		gtk_widget_destroy(dialog);
 	}
 }
+
+void remmina_main_show_warning_dialog(const gchar *message) {
+    GtkWidget *dialog;
+
+    if (remminamain->window) {
+        dialog = gtk_message_dialog_new(remminamain->window, GTK_DIALOG_MODAL, GTK_MESSAGE_WARNING, GTK_BUTTONS_CLOSE,
+                                        message, g_get_application_name());
+        gtk_dialog_run(GTK_DIALOG(dialog));
+        gtk_widget_destroy(dialog);
+    }
+}
